from threading import local
from attrdict import AttrDict

from uuid import uuid4

from contextlib import ContextDecorator

from security.config import settings


class SecurityLogger(ContextDecorator, local):

    loggers = []
    name = None
    store = True

    def __init__(self, id=None, related_objects=None, slug=None, data=None, extra_data=None):
        self.id = id or (uuid4() if self.name else None)
        self.parent = SecurityLogger.loggers[-1] if SecurityLogger.loggers else None
        self.related_objects = set(related_objects) if related_objects else set()
        self.slug = slug
        if self.parent:
            self.related_objects |= self.parent.related_objects
            if not self.slug:
                self.slug = self.parent.slug
        self.data = {}
        if data:
            self.data.update(data)
        self.parent_with_id = self._get_parent_with_id()
        self.extra_data = extra_data
        if self.extra_data is None:
            self.extra_data = self.parent.extra_data if self.parent else {}

        if self.store:
            SecurityLogger.loggers.append(self)
        self.backend_logs = AttrDict()

    def _get_parent_with_id(self):
        parent = self.parent
        while parent and not parent.id:
            parent = parent.parent
        return parent

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.store:
            self.close()

    def set_slug(self, slug):
        self.slug = slug

    def add_related_objects(self, *related_objects):
        self.related_objects |= set(related_objects)

    def update_extra_data(self, data):
        self.extra_data.update(data)

    def close(self):
        if not SecurityLogger.loggers or SecurityLogger.loggers[-1] != self:
            raise RuntimeError('Log already finished')
        else:
            SecurityLogger.loggers.pop()

    @property
    def release(self):
        return settings.RELEASE



# Generated by Django 3.1.13 on 2021-11-19 18:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('security_backends_sql', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='celerytaskinvocationlog',
            options={'ordering': ('-start',), 'verbose_name': 'celery task', 'verbose_name_plural': 'celery tasks'},
        ),
        migrations.AlterModelOptions(
            name='celerytaskrunlog',
            options={'ordering': ('-start',), 'verbose_name': 'celery task run', 'verbose_name_plural': 'celery tasks run'},
        ),
        migrations.AlterModelOptions(
            name='commandlog',
            options={'ordering': ('-start',), 'verbose_name': 'command log', 'verbose_name_plural': 'command logs'},
        ),
        migrations.AlterModelOptions(
            name='inputrequestlog',
            options={'ordering': ('-start',), 'verbose_name': 'input logged request', 'verbose_name_plural': 'input logged requests'},
        ),
        migrations.AlterModelOptions(
            name='outputrequestlog',
            options={'ordering': ('-start',), 'verbose_name': 'output logged request', 'verbose_name_plural': 'output logged requests'},
        ),
        migrations.AddField(
            model_name='celerytaskinvocationlog',
            name='release',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='release'),
        ),
        migrations.AddField(
            model_name='celerytaskrunlog',
            name='release',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='release'),
        ),
        migrations.AddField(
            model_name='celerytaskrunlog',
            name='waiting_time',
            field=models.FloatField(blank=True, null=True, verbose_name='waiting time'),
        ),
        migrations.AddField(
            model_name='commandlog',
            name='release',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='release'),
        ),
        migrations.AddField(
            model_name='inputrequestlog',
            name='release',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='release'),
        ),
        migrations.AddField(
            model_name='outputrequestlog',
            name='release',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='release'),
        ),
    ]
